﻿using DinerContracts.BindingModels;
using DinerContracts.BusinessLogicsContracts;
using DinerContracts.SearchModels;
using DinerContracts.StoragesContracts;
using DinerContracts.ViewModels;
using Microsoft.Extensions.Logging;

namespace DinerBusinessLogic.BusinessLogics
{
    public class SnackLogic : ISnackLogic
    {
        private readonly ILogger _logger;
        private readonly ISnackStorage _snackStorage;
        public SnackLogic(ILogger<SnackLogic> logger, ISnackStorage snackStorage)
        {
            _logger = logger;
            _snackStorage = snackStorage;
        }
        public List<SnackViewModel>? ReadList(SnackSearchModel? model)
        {
            _logger.LogInformation("ReadList. BouquetName:{BouquetName}.Id:{ Id}", model?.SnackName, model?.Id);
            var list = model == null ? _snackStorage.GetFullList() : _snackStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }
        public SnackViewModel? ReadElement(SnackSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. BouquetName:{BouquetName}.Id:{ Id}", model.SnackName, model.Id);
            var element = _snackStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }
        public bool Create(SnackBindingModel model)
        {
            CheckModel(model);
            if (_snackStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }
        public bool Update(SnackBindingModel model)
        {
            CheckModel(model);
            if (_snackStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
        public bool Delete(SnackBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_snackStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }
        private void CheckModel(SnackBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.SnackName))
            {
                throw new ArgumentNullException("Нет названия изделия",
               nameof(model.SnackName));
            }
            if (model.Price <= 0)
            {
                throw new ArgumentNullException("Цена изделия должна быть больше 0", nameof(model.Price));
            }
            _logger.LogInformation("Product. BouquetName:{BouquetName}.Cost:{ Cost}. Id: { Id}", model.SnackName, model.Price, model.Id);
            var element = _snackStorage.GetElement(new SnackSearchModel
            {
                SnackName = model.SnackName
            });
            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Изделие с таким названием уже есть");
            }
        }
    }
}
